searchData={"items":[{"type":"module","title":"Inertia","doc":"The Inertia.js adapter for Elixir/Phoenix applications.","ref":"Inertia.html"},{"type":"module","title":"Inertia.Controller","doc":"Controller functions for rendering Inertia.js responses.","ref":"Inertia.Controller.html"},{"type":"function","title":"Inertia.Controller.assign_prop/3","doc":"Assigns a prop value to the Inertia page data.","ref":"Inertia.Controller.html#assign_prop/3"},{"type":"function","title":"Inertia.Controller.inertia_always/1","doc":"Marks a prop value as \"always included\", which means it will be included in\nthe props on initial page load and subsequent partial loads (even when it's\nnot explicitly requested).","ref":"Inertia.Controller.html#inertia_always/1"},{"type":"function","title":"Inertia.Controller.inertia_lazy/1","doc":"Marks a prop value as lazy, which means it will only get evaluated if\nexplicitly requested in a partial reload.\n\nLazy props will _only_ be included the when explicitly requested in a partial\nreload. If you want to include the prop on first visit, you'll want to use a\nbare anonymous function or named function reference instead.\n\n```elixir\nconn\n# ALWAYS included on first visit...\n# OPTIONALLY included on partial reloads...\n# ALWAYS evaluated...\n|> assign_prop(:cheap_thing, cheap_thing())\n\n# ALWAYS included on first visit...\n# OPTIONALLY included on partial reloads...\n# ONLY evaluated when needed...\n|> assign_prop(:expensive_thing, fn -> calculate_thing() end)\n|> assign_prop(:another_expensive_thing, &calculate_another_thing/0)\n\n# NEVER included on first visit...\n# OPTIONALLY included on partial reloads...\n# ONLY evaluated when needed...\n|> assign_prop(:super_expensive_thing, inertia_lazy(fn -> calculate_thing() end))\n```","ref":"Inertia.Controller.html#inertia_lazy/1"},{"type":"function","title":"Inertia.Controller.render_inertia/3","doc":"Renders an Inertia response.","ref":"Inertia.Controller.html#render_inertia/3"},{"type":"type","title":"Inertia.Controller.always/0","doc":"","ref":"Inertia.Controller.html#t:always/0"},{"type":"type","title":"Inertia.Controller.lazy/0","doc":"","ref":"Inertia.Controller.html#t:lazy/0"},{"type":"module","title":"Inertia.HTML","doc":"HTML components for Inertia views.","ref":"Inertia.HTML.html"},{"type":"function","title":"Inertia.HTML.inertia_head/1","doc":"","ref":"Inertia.HTML.html#inertia_head/1"},{"type":"function","title":"Attributes - Inertia.HTML.inertia_head/1","doc":"* `content` (`:list`) (required) - The list of tags to inject into the `head` tag.","ref":"Inertia.HTML.html#inertia_head/1-attributes"},{"type":"function","title":"Inertia.HTML.inertia_page/1","doc":"","ref":"Inertia.HTML.html#inertia_page/1"},{"type":"function","title":"Attributes - Inertia.HTML.inertia_page/1","doc":"* `component` (`:string`) (required) - The name of the JavaScript page component.\n* `props` (`:map`) (required) - The page props (data).\n* `url` (`:string`) (required) - The page URL.\n* `version` (`:string`) (required) - The current asset version.","ref":"Inertia.HTML.html#inertia_page/1-attributes"},{"type":"function","title":"Inertia.HTML.inertia_ssr/1","doc":"","ref":"Inertia.HTML.html#inertia_ssr/1"},{"type":"function","title":"Attributes - Inertia.HTML.inertia_ssr/1","doc":"* `body` (`:string`) (required) - The server-rendered body.","ref":"Inertia.HTML.html#inertia_ssr/1-attributes"},{"type":"function","title":"Inertia.HTML.inertia_title/1","doc":"","ref":"Inertia.HTML.html#inertia_title/1"},{"type":"function","title":"Attributes - Inertia.HTML.inertia_title/1","doc":"* `prefix` (`:string`) - A prefix added before the content of `inner_block`. Defaults to `nil`.\n* `suffix` (`:string`) - A suffix added after the content of `inner_block`. Defaults to `nil`.","ref":"Inertia.HTML.html#inertia_title/1-attributes"},{"type":"function","title":"Slots - Inertia.HTML.inertia_title/1","doc":"* `inner_block` (required) - Content rendered inside the `title` tag.","ref":"Inertia.HTML.html#inertia_title/1-slots"},{"type":"module","title":"Inertia.Plug","doc":"The plug module for detecting Inertia.js requests.","ref":"Inertia.Plug.html"},{"type":"function","title":"Inertia.Plug.call/2","doc":"","ref":"Inertia.Plug.html#call/2"},{"type":"function","title":"Inertia.Plug.init/1","doc":"","ref":"Inertia.Plug.html#init/1"},{"type":"module","title":"Inertia.SSR","doc":"A supervisor that provides SSR support for Inertia views. This module is\nresponsible for starting a pool of Node.js processes that can run the SSR\nrendering function for your application.","ref":"Inertia.SSR.html"},{"type":"function","title":"Inertia.SSR.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Inertia.SSR.html#child_spec/1"},{"type":"function","title":"Inertia.SSR.start_link/1","doc":"Starts the SSR supervisor and accompanying Node.js workers.","ref":"Inertia.SSR.html#start_link/1"},{"type":"function","title":"Options - Inertia.SSR.start_link/1","doc":"- `:path` - (required) the path to the directory where your `ssr.js` file lives.\n- `:module` - (optional) the name of the Node.js module file. Defaults to \"ssr\".\n- `:pool_size` - (optional) the number of Node.js workers. Defaults to 4.","ref":"Inertia.SSR.html#start_link/1-options"},{"type":"exception","title":"Inertia.SSR.RenderError","doc":"","ref":"Inertia.SSR.RenderError.html"},{"type":"extras","title":"Inertia.js Phoenix Adapter","doc":"# Inertia.js Phoenix Adapter [![Hex Docs](https://img.shields.io/hexpm/v/inertia)](https://hexdocs.pm/inertia/readme.html)\n\nAn Elixir/Phoenix adapter for [Inertia.js](https://inertiajs.com/).","ref":"readme.html"},{"type":"extras","title":"Installation - Inertia.js Phoenix Adapter","doc":"The package can be installed by adding `inertia` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:inertia, \"~> 0.4.0\"}\n  ]\nend\n```\n\nAdd your desired configuration in your `config.exs` file:\n\n```elixir\n# config/config.exs\n\nconfig :inertia,\n  # The Phoenix Endpoint module for your application. This is used for building\n  # asset URLs to compute a unique version hash to track when something has\n  # changed (and a reload is required on the frontend).\n  endpoint: MyAppWeb.Endpoint,\n\n  # An optional list of static file paths to track for changes. You'll generally\n  # want to include any JavaScript assets that may require a page refresh when\n  # modified.\n  static_paths: [\"/assets/app.js\"],\n\n  # The default version string to use (if you decide not to track any static\n  # assets using the `static_paths` config). Defaults to \"1\".\n  default_version: \"1\",\n\n  # Enable server-side rendering for page responses (requires some additional setup,\n  # see instructions below). Defaults to `false`.\n  ssr: false,\n\n  # Whether to raise an exception when server-side rendering fails (only applies\n  # when SSR is enabled). Defaults to `true`.\n  #\n  # Recommended: enable in non-production environments and disable in production,\n  # so that SSR failures will not cause 500 errors (but instead will fallback to\n  # CSR).\n  raise_on_ssr_failure: true\n```\n\nThis library includes a few modules to help render Inertia responses:\n\n- [`Inertia.Plug`](https://hexdocs.pm/inertia/Inertia.Plug.html): a plug for detecting Inertia.js requests and preparing the connection accordingly.\n- [`Inertia.Controller`](https://hexdocs.pm/inertia/Inertia.Controller.html): controller functions for rendering Inertia.js-compatible responses.\n- [`Inertia.HTML`](https://hexdocs.pm/inertia/Inertia.HTML.html): HTML components for Inertia-powered views.\n\nTo get started, import `Inertia.Controller` in your controller helper and `Inertia.HTML` in your html helper:\n\n```diff\n  # lib/my_app_web.ex\n  defmodule MyAppWeb do\n    def controller do\n      quote do\n        use Phoenix.Controller, namespace: MyAppWeb\n\n+       import Inertia.Controller\n      end\n    end\n\n    def html do\n      quote do\n        use Phoenix.Component\n\n+       import Inertia.HTML\n      end\n    end\n  end\n```\n\nThen, install the plug in your browser pipeline:\n\n```diff\n  # lib/my_app_web/router.ex\n  defmodule MyAppWeb.Router do\n    use MyAppWeb, :router\n\n    pipeline :browser do\n      plug :accepts, [\"html\"]\n\n+     plug Inertia.Plug\n    end\n  end\n```\n\nNext, replace the title tag in your layout with the ` ` component, so that the client-side library will keep the title in sync, and add the ` ` component:\n\n```diff\n  # lib/my_app_web/components/layouts/root.html.heex\n   \n   \n     \n-        \n+        \n+      \n     \n```\n\nYou're now ready to start rendering inertia responses!","ref":"readme.html#installation"},{"type":"extras","title":"Rendering responses - Inertia.js Phoenix Adapter","doc":"Rendering an Inertia.js response looks like this:\n\n```elixir\ndefmodule MyAppWeb.ProfileController do\n  use MyAppWeb, :controller\n\n  def index(conn, _params) do\n    conn\n    |> assign_prop(:text, \"Hello world\")\n    |> render_inertia(\"ProfilePage\")\n  end\nend\n```\n\nThe `assign_prop` function allows you defined props that should be passed in to the component. The `render_inertia` function accepts the conn, the name of the component to render, and an optional map containing more initial props to pass to the page component.\n\nThis action will render an HTML page containing a ` ` element with the name of the component and the initial props, following Inertia.js conventions. On subsequent requests dispatched by the Inertia.js client library, this action will return a JSON response with the data necessary for rendering the page.","ref":"readme.html#rendering-responses"},{"type":"extras","title":"Lazy data evaluation - Inertia.js Phoenix Adapter","doc":"If you have expensive data for your props that may not always be required (that is, if you plan to use [partial reloads](https://inertiajs.com/partial-reloads)), you can wrap your expensive computation in a function and pass the function reference when setting your Inertia props. You may use either anonymous function (or named function reference), or the `inertia_lazy/1` function.\n\n> [!NOTE]\n> Lazy props will _only_ be included the when explicitly requested in a partial\n> reload. If you want to include the prop on first visit, you'll want to use a\n> bare anonymous function or named function reference instead. See below for\n> examples of how prop assignment behaves.\n\n```elixir\nconn\n# ALWAYS included on first visit...\n# OPTIONALLY included on partial reloads...\n# ALWAYS evaluated...\n|> assign_prop(:cheap_thing, cheap_thing())\n\n# ALWAYS included on first visit...\n# OPTIONALLY included on partial reloads...\n# ONLY evaluated when needed...\n|> assign_prop(:expensive_thing, fn -> calculate_thing() end)\n|> assign_prop(:another_expensive_thing, &calculate_another_thing/0)\n\n# NEVER included on first visit...\n# OPTIONALLY included on partial reloads...\n# ONLY evaluated when needed...\n|> assign_prop(:super_expensive_thing, inertia_lazy(fn -> calculate_thing() end))\n```","ref":"readme.html#lazy-data-evaluation"},{"type":"extras","title":"Shared data - Inertia.js Phoenix Adapter","doc":"To share data on every request, you can use the `Inertia.Controller.assign_prop/2` function inside of a `Plug.Conn` plug. For example, suppose you have a `UserAuth` plug responsible for fetching the currently-logged in user. Your plug might look something like this:\n\n```elixir\ndefmodule MyApp.UserAuth do\n  import Inertia.Controller\n  import Phoenix.Controller\n  import Plug.Conn\n\n  def authenticate_user(conn, _opts) do\n    user = get_user_from_session(conn)\n\n    conn\n    |> assign(:user, user)\n    |> assign_prop(:user, serialize_user(user))\n  end\n\n  # ...\nend\n```\n\nAnywhere this plug is used, the serialized `user` prop will be passed to the Inertia component.","ref":"readme.html#shared-data"},{"type":"extras","title":"Server-side rendering (Experimental) - Inertia.js Phoenix Adapter","doc":"The Inertia.js client library comes with with server-side rendering (SSR) support, which means you can have your Inertia-powered client hydrate HTML that has been pre-rendered on the server (instead of performing the initial DOM rendering).\n\n> [!NOTE]\n> The steps for enabling SSR in Phoenix are similar to other backend frameworks, but instead of running a separate Node.js server process to render HTML, this library spins up a pool of Node.js process workers to handle SSR calls. We'll highlight those differences below.\n\n#","ref":"readme.html#server-side-rendering-experimental"},{"type":"extras","title":"Add a server-side rendering module - Inertia.js Phoenix Adapter","doc":"To get started, you'll need to create a JavaScript module that exports a `render` function to perform the actual server-side rendering of pages. Suppose your `app.js` file looks something like this:\n\n```js\n// assets/js/app.js\n\nimport React from \"react\";\nimport { createInertiaApp } from \"@inertiajs/react\";\nimport { createRoot } from \"react-dom/client\";\nimport { pages } from \"./pages\";\n\ncreateInertiaApp({\n  resolve: (name) => {\n    return pages[name];\n  },\n  setup({ App, el, props }) {\n    createRoot(el).render( );\n  },\n});\n```\n\nCreate a second JavaScript file alongside your `app.js` called `ssr.js` with an exported `render` function.\n\n```js\n// assets/js/ssr.js\n\nimport React from \"react\";\nimport ReactDOMServer from \"react-dom/server\";\nimport { createInertiaApp } from \"@inertiajs/react\";\nimport { pages } from \"./pages\";\n\nexport function render(page) {\n  return createInertiaApp({\n    page,\n    render: ReactDOMServer.renderToString,\n    resolve: (name) => {\n      return pages[name];\n    },\n    setup: ({ App, props }) =>  ,\n  });\n}\n```\n\nThis is similar to the server entry-point [documented here](https://inertiajs.com/server-side-rendering#add-server-entry-point), except we are simply exporting a render function instead of creating a Node.js server process.\n\nNext, configure esbuild to compile the `ssr.js` bundle.\n\n```diff\n  # config/config.exs\n\n  config :esbuild,\n    version: \"0.21.4\",\n    app: [\n      args: ~w(\n        js/app.js\n        --bundle\n        --target=es2017\n        --outdir=../priv/static/assets\n        --external:/fonts/*\n        --external:/images/*\n      ),\n      cd: Path.expand(\"../assets\", __DIR__),\n      env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n    ],\n+   ssr: [\n+     args: ~w(\n+       js/ssr.js\n+       --bundle\n+       --platform=node\n+       --outdir=../priv\n+       --format=cjs\n+     ),\n+     cd: Path.expand(\"../assets\", __DIR__),\n+     env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n+   ]\n```\n\nAdd the `ssr` build step to the asset build and deploy scripts.\n\n```diff\n  # mix.exs\n\n  defp aliases do\n    [\n      setup: [\"deps.get\", \"ecto.setup\", \"assets.setup\", \"assets.build\"],\n      \"ecto.setup\": [\"ecto.create\", \"ecto.migrate\", \"run priv/repo/seeds.exs\"],\n      \"ecto.reset\": [\"ecto.drop\", \"ecto.setup\"],\n      test: [\"ecto.create --quiet\", \"ecto.migrate --quiet\", \"test\"],\n      \"assets.setup\": [\"tailwind.install --if-missing\", \"esbuild.install --if-missing\"],\n-     \"assets.build\": [\"tailwind app\", \"esbuild app\"],\n+     \"assets.build\": [\"tailwind app\", \"esbuild app\", \"esbuild ssr\"],\n      \"assets.deploy\": [\n        \"tailwind app --minify\",\n        \"esbuild app --minify\",\n+       \"esbuild ssr\",\n        \"phx.digest\"\n      ]\n    ]\n  end\n```\n\nAs configured, this will place the generated `ssr.js` bundle into the `priv` directory. Since it's generated code, add it to your `.gitignore` file.\n\n```diff\n  # .gitignore\n\n+ /priv/ssr.js\n```\n\n#","ref":"readme.html#add-a-server-side-rendering-module"},{"type":"extras","title":"Configuring your app for server-rendering - Inertia.js Phoenix Adapter","doc":"Now that you have a Node.js module capable of server-rendering your pages, let's tell the Inertia.js Phoenix library to use SSR.\n\nFirst, you'll need to add the `Inertia.SSR` module to your application supervision tree.\n\n```diff\n  # lib/my_app/application.ex\n\n  defmodule MyApp.Application do\n    use Application\n\n    @impl true\n    def start(_type, _args) do\n      children = [\n        MyAppWeb.Telemetry,\n        MyApp.Repo,\n        {DNSCluster, query: Application.get_env(:MyApp, :dns_cluster_query) || :ignore},\n        {Phoenix.PubSub, name: MyApp.PubSub},\n        # Start the Finch HTTP client for sending emails\n        {Finch, name: MyApp.Finch},\n        # Start a worker by calling: MyApp.Worker.start_link(arg)\n        # {MyApp.Worker, arg},\n\n+       # Start the SSR process pool\n+       # You must specify a `path` option to locate the directory where the `ssr.js` file lives.\n+       {Inertia.SSR, path: Path.join([Application.app_dir(:my_app), \"priv\"])}\n\n        # Start to serve requests, typically the last entry\n        MyAppWeb.Endpoint,\n      ]\n```\n\nThen, update your Inertia Elixir configuration to enable SSR.\n\n```diff\n  # config/config.exs\n\n  config :inertia,\n    # The Phoenix Endpoint module for your application. This is used for building\n    # asset URLs to compute a unique version hash to track when something has\n    # changed (and a reload is required on the frontend).\n    endpoint: MyAppWeb.Endpoint,\n\n    # An optional list of static file paths to track for changes. You'll generally\n    # want to include any JavaScript assets that may require a page refresh when\n    # modified.\n    static_paths: [\"/assets/app.js\"],\n\n    # The default version string to use (if you decide not to track any static\n    # assets using the `static_paths` config). Defaults to \"1\".\n    default_version: \"1\",\n\n    # Enable server-side rendering for page responses (requires some additional setup, \n    # see instructions below). Defaults to `false`.\n-   ssr: false\n+   ssr: true\n\n    # Whether to raise an exception when server-side rendering fails (only applies\n    # when SSR is enabled). Defaults to `true`.\n    #\n    # Recommended: enable in non-production environments and disable in production,\n    # so that SSR failures will not cause 500 errors (but instead will fallback to\n    # CSR).\n    raise_on_ssr_failure: true\n```\n\n#","ref":"readme.html#configuring-your-app-for-server-rendering"},{"type":"extras","title":"Client side hydration - Inertia.js Phoenix Adapter","doc":"[Follow the instructions from the Inertia.js docs](https://inertiajs.com/server-side-rendering#client-side-hydration) for updating your client-side code to hydrate the pre-rendered HTML coming from the server.\n\n---\n\nMaintained by the team at [SavvyCal](https://savvycal.com)","ref":"readme.html#client-side-hydration"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.4.0 - Changelog","doc":"- Support for partial reloads ([#6](https://github.com/svycal/inertia-phoenix/issues/6))\n- Support lazy data evaluation ([#7](https://github.com/svycal/inertia-phoenix/issues/7))","ref":"changelog.html#0-4-0"},{"type":"extras","title":"0.3.0 - Changelog","doc":"- Add `raise_on_ssr_failure` configuration","ref":"changelog.html#0-3-0"},{"type":"extras","title":"0.2.0 - Changelog","doc":"- Add SSR support\n- Add ` ` component for rendering head elements provided by SSR","ref":"changelog.html#0-2-0"},{"type":"extras","title":"0.1.0 - Changelog","doc":"- Initial release","ref":"changelog.html#0-1-0"},{"type":"extras","title":"License","doc":"# License\n\nMIT License\n\nCopyright (c) 2024 SavvyCal, Inc.\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}